function Y = RoundObjectives (Y_Exact, rounding_lvl)
% User defines rounding_lvl that is a vector variable with the desired rounding level, e.g. [0.1, 0.01, 10, 1000] for a four-objective optimization problem.
% This function takes as input the vector variable Y_Exact that has the exact value of the objective functions and returns in Y their rounded value according to the rounding_lvl.
% Length of input vectors should be the same
if length(Y_Exact) ~= length(rounding_lvl)
    error('Error: input vector variables Y_Exact & rounding_lvl should have the same size');
else
    % Round each objective function to desired precision levels.
    Y = rounding_lvl .* round (Y_Exact./ rounding_lvl);
end
   